#include "kconnect.hxx"
#include "kcmd.h"
#include "ikcmd.h"
#include "ikterm.h"

extern "C" {
#include "ckuusr.h"
}

static const int TYPE_TCPIP     = 0;
static const int TYPE_DIAL      = 1;
static const int TYPE_DIRECT    = 2;
static const int TYPE_SUPERLAT  = 3;
static const int NUM_TYPES      = 4;

static const int NUM_CONTROLS = 13;
static int controls[NUM_CONTROLS] = {
      IDC_COMBOCONNECTTYPE
    , IDC_STATICONE
    , IDC_EDITONE
    , IDC_STATICTWO
    , IDC_EDITTWO
    , IDC_GROUPLOGIN
    , IDC_STATICLOGINNAME
    , IDC_STATICLOGINPASSWORD
    , IDC_EDITLOGINNAME
    , IDC_EDITLOGINPASSWORD
    , IDC_STATICGROUP
    , IDC_RADIOONE
    , IDC_RADIOTWO
};
    

static k_cell cmdarray[][NUM_CONTROLS] = {
{{TYPE_TCPIP,"tcp/ip"}, {1,"Host:"}, {1,0}, {1,"Port:"}, {1,"telnet"}, {1,"Login"}, {1,"Name:"}, {1,"Password:"}, {1,0}, {1,0}, {1,""}, {2,"telnet"}, {1,"rlogin"} },
{{TYPE_DIAL,"dial" },    {1,"Telephone Number:"}, {1,0} },
{{TYPE_DIRECT,"direct"}, {0,0} },
{{TYPE_SUPERLAT,"superlat"}, {1,"Service:"}, {1,0}, {0,0}, {0,0}, {0,0}, {0,0}, {0,0}, {0,0}, {0,0}, {1,""}, {2,"service"}, {1,"node/port"} }
};


static KConnect* kconnect = 0;
/*------------------------------------------------------------------------
------------------------------------------------------------------------*/
extern "C" {
BOOL CALLBACK ConnectDlgProc( HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam )
{
    return (BOOL)kconnect->message( hwnd, msg, wParam, lParam );
}
}

/*------------------------------------------------------------------------
------------------------------------------------------------------------*/
KConnect::KConnect( K_GLOBAL* g, KWin* p )
    : KWin( g )
{
    hcombo = 0;
    parent = p;
    kconnect = this;
}

/*------------------------------------------------------------------------
------------------------------------------------------------------------*/
KConnect::~KConnect()
{
    kconnect = 0;
}

/*------------------------------------------------------------------------
------------------------------------------------------------------------*/
void KConnect::show( Bool bVisible )
{
    DialogBox( hInst
        , MAKEINTRESOURCE(IDD_CONNECT)
        , parent->hwnd()
        , ConnectDlgProc );
}

/*------------------------------------------------------------------------
------------------------------------------------------------------------*/
void KConnect::initDialog( HWND hwnd )
{
    hWnd = hwnd;
    k_cell* p;
    hcombo = GetDlgItem( hwnd, IDC_COMBOCONNECTTYPE );
    for( int i = 0; i < NUM_TYPES; i++ )
    {
        p = &(cmdarray[i][0]);
        SendMessage( hcombo, CB_ADDSTRING, 0, (LPARAM)(LPSTR) p->text );
    }
    SendMessage( hcombo, CB_SETCURSEL, (WPARAM)0, (LPARAM)0 );
    switchType();
}

/*------------------------------------------------------------------------
------------------------------------------------------------------------*/
void KConnect::switchType()
{
    int sel = SendMessage( hcombo, CB_GETCURSEL, 0, 0 );
    HWND hctrl;
    Bool visible;
    k_cell* p;
    for( int i = 1; i < NUM_CONTROLS; i++ )
    {
        p = &(cmdarray[sel][i]);
        hctrl = GetDlgItem( hWnd, controls[i] );
        visible = IsWindowVisible( hctrl );
        if( p->action ) {
            if( !visible )
                ShowWindow( hctrl, TRUE );
            if( p->text )
                SetWindowText( hctrl, p->text );
            else
                SetWindowText( hctrl, "" );
            if( p->action == 2 )
                SendMessage( hctrl, BM_SETCHECK, BST_CHECKED, NULL );
            else if( controls[i] == IDC_RADIOONE || controls[i] == IDC_RADIOTWO )
                SendMessage( hctrl, BM_SETCHECK, BST_UNCHECKED, NULL );
        }       
        else {
            if( visible )
                ShowWindow( hctrl, FALSE );
        }
    }
}

/*------------------------------------------------------------------------
------------------------------------------------------------------------*/
void KConnect::toggleRadio( long id )
{
    int sel = SendMessage( hcombo, CB_GETCURSEL, 0, 0 );
    switch( sel )
    {
    case TYPE_TCPIP:
        {
            HWND hedit = GetDlgItem( hWnd, IDC_EDITTWO );
            if( id == IDC_RADIOONE ) {
                SetWindowText( hedit, "telnet" );
            }
            else if( id == IDC_RADIOTWO ) {
                SetWindowText( hedit, "login" );
            }
            break;
        }
    case TYPE_SUPERLAT:
        {	
            HWND hlabel1 = GetDlgItem( hWnd, IDC_STATICONE );
            HWND hedit1 = GetDlgItem( hWnd, IDC_EDITONE );
            HWND hlabel2 = GetDlgItem( hWnd, IDC_STATICTWO );
            HWND hedit2 = GetDlgItem( hWnd, IDC_EDITTWO );
            if( id == IDC_RADIOONE ) {
                SetWindowText( hlabel1, "Service:" );
                SetWindowText( hedit1, "" );

                ShowWindow( hlabel1, TRUE );
                ShowWindow( hedit1, TRUE );
                ShowWindow( hlabel2, FALSE );
                ShowWindow( hedit2, FALSE );
            }
            else if( id == IDC_RADIOTWO ) {
                SetWindowText( hlabel1, "Node:" );
                SetWindowText( hedit1, "" );
                SetWindowText( hlabel2, "Port:" );
                SetWindowText( hedit2, "" );

                ShowWindow( hlabel1, TRUE );
                ShowWindow( hedit1, TRUE );
                ShowWindow( hlabel2, TRUE );
                ShowWindow( hedit2, TRUE );
            }
            break;
        }
    default:
        return;	// error
    }

}       
                    
/*------------------------------------------------------------------------
------------------------------------------------------------------------*/
void KConnect::makeConnection( HWND hwnd )
{
    int sel = SendMessage( hcombo, CB_GETCURSEL, 0, 0 );
    switch( sel )
    {
    case TYPE_TCPIP: {
	HWND hhost = GetDlgItem( hwnd, IDC_EDITONE );
	HWND hport = GetDlgItem( hwnd, IDC_EDITTWO );
	char host[128];
	char port[128];
	GetWindowText( hhost, host, 128 );
	GetWindowText( hport, port, 128 );
        EndDialog( hwnd, TRUE );
        cx_net(NET_TCPB,0,host,port,NULL,NULL,NULL,-1,-1,1,1,0,1,1);
	break;
    }
    case TYPE_DIAL: {
	HWND hedit = GetDlgItem( hwnd, IDC_EDITONE );
	char text[128];
	GetWindowText( hedit, text, 128 );
        EndDialog( hwnd, TRUE );
	break;
    }
    case TYPE_DIRECT:
        EndDialog( hwnd, TRUE );
	break;
    case TYPE_SUPERLAT: {
	HWND hhost = GetDlgItem( hwnd, IDC_EDITONE );
	char host[128];
	GetWindowText( hhost, host, 128 );
        EndDialog( hwnd, TRUE );
        cx_net(NET_SLAT,0,host,"",NULL,NULL,NULL,-1,0,0,1,0,1,1);
	break;
    }
    default:
        EndDialog( hwnd, TRUE );
    }
}

/*------------------------------------------------------------------------
------------------------------------------------------------------------*/
Bool KConnect::message( HWND hwnd, UINT msg, UINT wParam, LONG lParam )
{
    Bool done = FALSE;
    switch( msg )
    {
    case WM_INITDIALOG:
        initDialog( hwnd );
        break;

    case WM_COMMAND: {
        long id = (short) LOWORD(wParam);
        long code = (short) HIWORD(wParam);
        switch( id )
        {
        case IDCANCEL:
            EndDialog( hwnd, TRUE );
            break;

        case IDC_COMBOCONNECTTYPE:
            if( code == CBN_SELCHANGE )
                switchType();
            break;

        case IDOK:
            makeConnection(hwnd);
            break;

        case IDC_RADIOONE:
        case IDC_RADIOTWO:
            toggleRadio( id );
            break;
        }
        done = TRUE;
        break;
    }

    case WM_CLOSE:
        EndDialog( hwnd, TRUE );
        done = TRUE;
        break;
    }
    return done;
}

